/*
 * java applet for user interface to the scene
 * Jaroslav Semancik and Filip Vujtech, 2003
 */

import java.awt.*;
import java.applet.*;
import vrml.eai.*;
import vrml.eai.event.*;
import vrml.eai.field.*;


public class applet extends Applet implements VrmlEventListener
{
    Button lightBut;                                // GUI elements
    Label buttonLab, distLab, distvalLab,
          seesawLab, xseesawLab, zseesawLab,
          snowLab, xsnowLab, zsnowLab;
    TextField xseesawTF, zseesawTF,
              xsnowTF, zsnowTF;
    Scrollbar xseesawScroll, zseesawScroll,
              xsnowScroll, zsnowScroll;

    final static int RANGE = 2;                     // moving in VRML by +-3 units
    final static int SBRES = 100;                   // scrollbar values 0-99

    EventOutSFBool isON;                            // VRML world elements
    EventOutSFVec3f position;
    Browser browser;
    Node ctrl_script, light, prox, seesaw, snow;

    float ap[],                       // avatar position related to proximity sensor
          ossp[] = { 5f, 0f, -1f },   // original seesaw and snowman position
          osp[] = { -5f, 0f, -4f },   // in absolotute world coordinates
          rssp[] = { 0f, 0f, 0f },    // current relative seesaw and snowman position
          rsp[] = { 0f, 0f, 0f };     // related to its original position

    public void init()
    {
        super.init();

        // GUI layout
        setBackground(new Color(150, 150, 150));
        setLayout(null);

        // create GUI elements
        buttonLab = new Label("Turn headlight:", Label.RIGHT);
        lightBut = new Button("ON");
        distLab = new Label("", Label.RIGHT);
        distvalLab = new Label();

        seesawLab = new Label("See-saw position:", Label.LEFT);
        xseesawLab = new Label("X");
        zseesawLab = new Label("Z");
        xseesawTF = new TextField((new Float(ossp[0])).toString());
        zseesawTF = new TextField((new Float(ossp[2])).toString());
        xseesawScroll = new Scrollbar(Scrollbar.HORIZONTAL, SBRES / 2, 1, 0, SBRES);
        zseesawScroll = new Scrollbar(Scrollbar.HORIZONTAL, SBRES / 2, 1, 0, SBRES);

        snowLab = new Label("Snowman position:", Label.LEFT);
        xsnowLab = new Label("X");
        zsnowLab = new Label("Z");
        xsnowTF = new TextField((new Float(osp[0])).toString());
        zsnowTF = new TextField((new Float(osp[2])).toString());
        xsnowScroll = new Scrollbar(Scrollbar.HORIZONTAL, SBRES / 2, 1, 0, SBRES);
        zsnowScroll = new Scrollbar(Scrollbar.HORIZONTAL, SBRES / 2, 1, 0, SBRES);

        // position GUI elements
        buttonLab.setBounds(100, 6, 90, 20);
        lightBut.setBounds(200, 6, 40, 20);
        distLab.setBounds(10, 31, 180, 20);
        distvalLab.setBounds(200, 31, 80, 20);

        seesawLab.setBounds(300, 6, 100, 20);
        xseesawLab.setBounds(430, 6, 15, 20);
        xseesawTF.setBounds(445, 7, 30, 17);
        zseesawLab.setBounds(520, 6, 15, 20);
        zseesawTF.setBounds(535, 7, 30, 17);
        xseesawScroll.setBounds(420, 25, 70, 10);
        zseesawScroll.setBounds(510, 25, 70, 10);

        snowLab.setBounds(300, 43, 100, 20);
        xsnowLab.setBounds(430, 43, 15, 20);
        xsnowTF.setBounds(445, 44, 30, 17);
        zsnowLab.setBounds(520, 43, 15, 20);
        zsnowTF.setBounds(535, 44, 30, 17);
        xsnowScroll.setBounds(420, 62, 70, 10);
        zsnowScroll.setBounds(510, 62, 70, 10);

        // add GUI elements
        add(buttonLab);
        add(lightBut);
        add(distLab);
        add(distvalLab);

        add(seesawLab);
        add(xseesawLab);
        add(xseesawTF);
        add(zseesawLab);
        add(zseesawTF);
        add(xseesawScroll);
        add(zseesawScroll);

        add(snowLab);
        add(xsnowLab);
        add(xsnowTF);
        add(zsnowLab);
        add(zsnowTF);
        add(xsnowScroll);
        add(zsnowScroll);
    }


    public void start()
    {
        super.start();

        browser = BrowserFactory.getBrowser(this);
        ctrl_script = browser.getNode("OVL_SCRIPT");
        light = browser.getNode("LIGHT_BUTTON");
        prox = browser.getNode("PS");
        seesaw = browser.getNode("HOUP");
        snow = browser.getNode("SNOW");

        isON = (EventOutSFBool) light.getEventOut("invertstate");
        isON.setUserData(new Integer(1));
        isON.addVrmlEventListener(this);

        position = (EventOutSFVec3f) prox.getEventOut("position_changed");
        position.setUserData(new Integer(2));
        position.addVrmlEventListener(this);

        setPosition();
    }


    public void paint(Graphics g)
    {
        drawRainbowBorder(g);
        g.setColor(Color.black);
    }


    private void drawRainbowBorder(Graphics g)
    {
        float t;

        // draw rainbow border
        for (int i = 0; i<=5; i++)
        {
            t = i / 5.0f;
            g.setColor(new Color((char)(147*(1-t) + 150*t),
                                 (char)(95*(1-t) + 150*t),
                                 (char)(66*(1-t)+150*t)));
            g.fillRect(i*2, i, 600 - i*4, 80 - i*2);
        }
    }


    public void eventOutChanged(VrmlEvent event)
    {
        int i = ( (Integer) event.getData() ).intValue();
        switch(i)
	{
            // light
            case 1:
                if (((EventOutSFBool) event.getSource()).getValue())
                    lightBut.setLabel("OFF");
                else
                    lightBut.setLabel("ON");
                break;

            // avatar
            case 2:
                calcDist();
                break;

            default:
	        break;
	}
    }


    // button and textfield events

    public boolean action(Event event, Object what)
    {
        // tlacitko
        if(event.target instanceof Button)
        {
            ((EventInSFTime) (ctrl_script.getEventIn("touchTime"))).setValue(0);
            return true;
        }

        // text field
        else if(event.target instanceof TextField)
        {
            if (event.target == xseesawTF)
            {
                rssp[0] = java_atof(xseesawTF.getText()) - ossp[0];
                xseesawScroll.setValue(VRML2Scroll(rssp[0]));
            }

            if (event.target == zseesawTF)
            {
                rssp[2] = java_atof(zseesawTF.getText()) - ossp[2];
                zseesawScroll.setValue(VRML2Scroll(rssp[2]));
            }

            if (event.target == xsnowTF)
            {
                rsp[0] = java_atof(xsnowTF.getText()) - osp[0];
                xsnowScroll.setValue(VRML2Scroll(rsp[0]));
            }

            if (event.target == zsnowTF)
            {
                rsp[2] = java_atof(zsnowTF.getText()) - osp[2];
                zsnowScroll.setValue(VRML2Scroll(rsp[2]));
            }

            setPosition();
            return true;
	}

        return super.action(event,what); 
    }


    //scrollbar events

    public boolean handleEvent(Event event)
    {
        if (event.target instanceof Scrollbar)
        {
            if (event.target == xseesawScroll)
            {
                rssp[0] = Scroll2VRML(xseesawScroll.getValue());
                xseesawTF.setText((new Float(rssp[0] + ossp[0])).toString());
            }

            if (event.target == zseesawScroll)
            {
                rssp[2] = Scroll2VRML(zseesawScroll.getValue());
                zseesawTF.setText((new Float(rssp[2] + ossp[2])).toString());
            }

            if (event.target == xsnowScroll)
            {
                rsp[0] = Scroll2VRML(xsnowScroll.getValue());
                xsnowTF.setText((new Float(rsp[0] + osp[0])).toString());
            }

            if (event.target == zsnowScroll)
            {
                rsp[2] = Scroll2VRML(zsnowScroll.getValue());
                zsnowTF.setText((new Float(rsp[2] + osp[2])).toString());
            }

            setPosition();
            return true;
        }

        return super.handleEvent(event);
    }


    // calculate avatar's distances to objects

    private void calcDist()
    {
        // get avatar position
        EventOutSFVec3f pc = (EventOutSFVec3f) prox.getEventOut("position_changed");
        ap = pc.getValue();

        // distance to see-saw (proximity sensor is centered at see-saw)
        float dss = (float) Math.sqrt(
                  (rssp[0] - ap[0]) * (rssp[0] - ap[0])
//              + (rssp[1] - ap[1]) * (rssp[1] - ap[1])
                + (rssp[2] - ap[2]) * (rssp[2] - ap[2]));

        // vector from proximity sensor center (see-saw) to snowman
        float dp[] = new float[3];
        dp[0] = osp[0] - ossp[0];
        dp[1] = osp[1] - ossp[1];
        dp[2] = osp[2] - ossp[2];

        // distance to snowman
        float ds = (float) Math.sqrt(
                  (dp[0] + rsp[0] - ap[0]) * (dp[0] + rsp[0] - ap[0])
//              + (dp[1] + rsp[1] - ap[1]) * (dp[1] + rsp[1] - ap[1])
                + (dp[2] + rsp[2] - ap[2]) * (dp[2] + rsp[2] - ap[2]));

        if (dss < 6 || ds < 6)
        {
            if (dss < ds)
            {
                distLab.setText("You are near the see-saw:");
                distvalLab.setText("" + dss);
            }
            else
            {
                distLab.setText("You are near the snowman:");
                distvalLab.setText("" + ds);
            }
        }
        else
        {
            distLab.setText("");
            distvalLab.setText("");
        }
    }


    // set position of objects in VRML scene

    private void setPosition()
    {
        EventInSFVec3f ss_pos = (EventInSFVec3f) seesaw.getEventIn("poloha");
        EventInSFVec3f l_pos = (EventInSFVec3f) snow.getEventIn("pos");
        ss_pos.setValue(rssp);
        l_pos.setValue(rsp);
        calcDist();
    }


    // VRML to Scrollbar range conversion

    private int VRML2Scroll(float f)
    {
        float t = f + RANGE;
        if (t < 0) t = 0;
        if (t > 2 * RANGE) t = 2 * RANGE;
        return (int)(t / (2 * RANGE) - 0.0001 * SBRES);
    }


    // Scrollbar to VRML range conversion

    private float Scroll2VRML(int i)
    {
        return (float)i / (SBRES - 1) * RANGE * 2 - RANGE;
    }


    private float java_atof(String str)
    {
        Float F;

        try
        {
            F = new Float(str);
        }
        catch(Exception e)
        {
            F = new Float(0.0);
        } 

        return F.floatValue();
    }

}
